class _Config(object):
    _message_introspection = True

    def is_message_introspection_enabled(self):
        return self._message_introspection

    def enable_message_introspection(self):
        self._message_introspection = True

    def disable_message_introspection(self):
        self._message_introspection = False

conf = _Config()



import sys

from contextlib import contextmanager
from .conf import conf
from .rewrite import AssertionRewritingHook


@contextmanager
def rewrite_assertions_context():
    hook = AssertionRewritingHook()
    sys.meta_path.insert(0, hook)
    try:
        yield
    finally:
        sys.meta_path.remove(hook)


def disable_message_introspection():
    conf.disable_message_introspection()

def enable_message_introspection():
    conf.enable_message_introspection()



"""
Script used by tox.ini to check the manifest file if we are under version control, or skip the
check altogether if not.

"check-manifest" will needs a vcs to work, which is not available when testing the package
instead of the source code (with ``devpi test`` for example).
"""

from __future__ import print_function

import os
import subprocess
import sys


if os.path.isdir('.git'):
    sys.exit(subprocess.call('check-manifest', shell=True))
else:
    print('No .git directory found, skipping checking the manifest file')
    sys.exit(0)




from _pytest.assertion.rewrite import AssertionRewritingHook as PytestRewriteHook
import os
import shutil
import sys
from contextlib import contextmanager
from tempfile import mkdtemp

import emport

import dessert
import pytest


def test_dessert(module):
    with pytest.raises(AssertionError) as error:
        module.func()

    assert 'dessert*' in str(error.value)
    assert "where" in str(error.value)
    assert "+" in str(error.value)


def test_disable_introspection(add_assert_message, module, assert_message):
    with _disable_introspection():
        with pytest.raises(AssertionError) as error:
            module.func()
        if not add_assert_message:
            assert 'dessert*' in str(error.value)
            assert "where" in str(error.value)
            assert "+" in str(error.value)
        else:
            assert assert_message in str(error.value)
            assert "+" not in str(error.value)


def test_warnings_from_rewrite(source_filename):
    tmp_dir = os.path.dirname(source_filename)
    full_path = os.path.join(tmp_dir, 'file_with_warnings.py')
    with open(full_path, "w") as f:
        f.write(r"""
import warnings
warnings.simplefilter('always')
warnings.warn('Some import warning')

def func():
    assert True
""")
    with dessert.rewrite_assertions_context():
        with _disable_pytest_rewriting():
            with pytest.warns((UserWarning)) as caught:
                emport.import_file(full_path)
            [warning] = caught.list
            assert warning.filename == full_path


@pytest.fixture(scope='session', autouse=True)
def mark_dessert():
    # pylint: disable=protected-access
    assert not dessert.rewrite._MARK_ASSERTION_INTROSPECTION
    dessert.rewrite._MARK_ASSERTION_INTROSPECTION = True

@pytest.fixture
def module(request, source_filename):
    with dessert.rewrite_assertions_context():
        with _disable_pytest_rewriting():
            module = emport.import_file(source_filename)

    @request.addfinalizer
    def drop_from_sys_modules():  # pylint: disable=unused-variable
        sys.modules.pop(module.__name__)

    return module

@contextmanager
def _disable_pytest_rewriting():
    old_meta_path = sys.meta_path[:]
    try:
        for index, plugin in reversed(list(enumerate(sys.meta_path))):
            if isinstance(plugin, PytestRewriteHook):
                sys.meta_path.pop(index)
        yield
    finally:
        sys.meta_path[:] = old_meta_path

@contextmanager
def _disable_introspection():
    dessert.disable_message_introspection()
    try:
        yield
    finally:
        dessert.enable_message_introspection()


@pytest.fixture(params=[
    "assert x() + y()",
    "assert f(1) > g(100)",
    "assert f(g(2)) == f(g(1))",
    ])
def assertion_line(request):
    return request.param


@pytest.fixture(params=[True, False])
def add_assert_message(request):
    return request.param

@pytest.fixture
def assert_message(request):
    return 'msg'


@pytest.fixture
def source(assertion_line, add_assert_message, assert_message):
    if add_assert_message:
        assertion_line += ", '{}'".format(assert_message)
    returned = """def f(x):
    return x

x = lambda: 1
y = lambda: -1

g = h = f

def func():
    variable = False

    {0}
    """.format(assertion_line)
    return returned


@pytest.fixture
def source_filename(request, source):
    path = mkdtemp()

    @request.addfinalizer
    def delete():  # pylint: disable=unused-variable
        shutil.rmtree(path)

    filename = os.path.join(path, "sourcefile.py")
    with open(filename, "w") as f:
        f.write(source)

    return filename



from __future__ import print_function

import os
import shutil
import functools
import pytest
import subprocess
import sys


def test_example_code_simple_assertion(run_example):
    assert run_example('no_message_simple_equality') == 'assert 1 == 2'


def test_example_message_assertion(run_example):
    assert run_example('with_message_simple_equality') == 'Here is an assertion message\nassert 1 == 2'



@pytest.fixture
def run_example(tmpdir):
    here = os.path.dirname(__file__)
    shutil.copy(os.path.join(here, 'driver.py'), str(tmpdir))
    shutil.copy(os.path.join(here, 'examples.py'), str(tmpdir))

    return functools.partial(_run, str(tmpdir.join('driver.py')))

def _run(filename, example_name):
    executable = sys.executable
    output = subprocess.check_output([executable, filename, example_name], stderr=subprocess.STDOUT).decode('utf-8')
    return output.strip()



import doctest
import os
import shutil

import pytest
import tempfile


def test_readme_doctests(tmp_filename):
    readme_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "README.md"))
    assert os.path.exists(readme_path)
    result = doctest.testfile(readme_path, module_relative=False, globs={"tmp_filename": tmp_filename})
    assert result.failed == 0

@pytest.fixture
def tmp_filename(request):
    tmp_dir = tempfile.mkdtemp()
    @request.addfinalizer
    def delete():
        shutil.rmtree(tmp_dir)
    return os.path.join(tmp_dir, "filename.py")



import sys

if __name__ == "__main__":
    [_, example_name] = sys.argv # pylint: disable=unbalanced-tuple-unpacking
    import dessert

    with dessert.rewrite_assertions_context():
        import examples

    func = getattr(examples, example_name)
    try:
        func()
    except AssertionError as e:
        print(e)
    else:
        sys.exit('Assertion not matched')



# pylint: disable=unused-import,import-error
import fnmatch
import os
import sys
from os.path import sep
from posixpath import sep as posix_sep


if sys.version_info[:2] >= (3, 6):
    from pathlib import Path, PurePath
else:
    from pathlib2 import Path, PurePath


def fnmatch_ex(pattern, path):
    """FNMatcher port from py.path.common which works with PurePath() instances.

    The difference between this algorithm and PurePath.match() is that the latter matches "**" glob expressions
    for each part of the path, while this algorithm uses the whole path instead.

    For example:
        "tests/foo/bar/doc/test_foo.py" matches pattern "tests/**/doc/test*.py" with this algorithm, but not with
        PurePath.match().

    This algorithm was ported to keep backward-compatibility with existing settings which assume paths match according
    this logic.

    References:
    * https://bugs.python.org/issue29249
    * https://bugs.python.org/issue34731
    """

    path = PurePath(path)
    iswin32 = sys.platform.startswith("win")

    if iswin32 and sep not in pattern and posix_sep in pattern:
        # Running on Windows, the pattern has no Windows path separators,
        # and the pattern has one or more Posix path separators. Replace
        # the Posix path separators with the Windows path separator.
        pattern = pattern.replace(posix_sep, sep)

    if sep not in pattern:
        name = path.name
    else:
        name = str(path)
        if path.is_absolute() and not os.path.isabs(pattern):
            pattern = "*{}{}".format(os.sep, pattern)
    return fnmatch.fnmatch(name, pattern)
